// PROGRAMA: SEGUIDOR.C
// Interface para comunicao serial que permite o envio e recebimento caracteres individuais.
// Autor: Roberto Vichinsky
// IMPORTANTE: Este cdigo fonte deve ser gravado como um programa C puro, portanto,
//             na janela de gravao do DEVC++ escolha o tipo "C source file (*.C)" para
//             que o programa seja gravado com o sufixo ".c".
// ---------------------------------------------------------------------------------------------

#include <conio.h>
#include <stdio.h>
#include <windows.h>
#include <time.h>
#include <locale.h>
#include <stdlib.h>

// Prottipo das funes utilizadas
// --------------------------------
HANDLE serial_abre (int porta, const long int BaudRate, const char parity, const char data); // manipulador da porta serial (abertura)
void serial_limpa(const int porta);                     // Limpeza do buffer
void serial_fecha(const int porta);                     // Fechamento da porta serial
char serial_recebe(const int porta);                    // Leitura da porta serial (recebimento)
void serial_envia(const int porta, const int txchar);   // Transmisso de caractere pela porta serial
void delay_ms(clock_t millis);                          // Temporizador (timer)

HANDLE hCom; //Define hCom como manipulador (ser usado como referncia  porta serial)

int main() {
	clock_t timeout;
	unsigned char r;
	unsigned char e;
	unsigned char tecla;
	unsigned char para='7';
	int porta;
	int taxa;
	setlocale(LC_ALL,"Portuguese");
	system("color F1");
    system("cls");
    printf(" MONITOR SERIAL - Veculo Seguidor de Linhas\n");
    printf(" -------------------------------------------\n");
    printf(" Nmero da porta serial: ");scanf("%d",&porta);
	printf(" Taxa de transmisso...: ");scanf("%d",&taxa);
	if(!serial_abre(porta ,taxa, '8', 'N')) {  
		getch(); system("cls");  exit(1);
	}
	timeout = clock() + 60000;  // 60 segundos
	printf("\n COMANDOS:\n");
	printf(" [L].....Liga o veculo         [D]......Desliga o veculo \n");
	printf(" [R].....Modo controle remoto   [S]......Modo seguidor de linha\n");
	printf(" [UP]....Move para frente       [DOWN]...Parada (desliga motores)\n");
	printf(" [LEFT]..Curva  esquerda       [RIGHT]..Curva  direita\n\n");
	while(1) {
		r=serial_recebe(porta);printf(" Recebido: %c\r",r);
		if(r!='\0') {
			printf(" Recebido: %c\r",r);
		}
		if(kbhit()) {
			tecla=getch();
			if(tecla=='*') {goto fim;}
			if (tecla=='D' || tecla=='d') {tecla='0';}  // Desliga
			if (tecla=='L' || tecla=='l') {tecla='1';}  // Liga
			if (tecla=='S' || tecla=='s') {tecla='2';}  // Seguidor de linha
			if (tecla=='R' || tecla=='r') {tecla='3';}  // Controle remoto
			if (tecla=='H') {tecla='4';}                // Frente (seta para cima)
			if (tecla=='M') {tecla='5';}                // Esquerda (seta para esquerda)
			if (tecla=='K') {tecla='6';}                // Direita (seta para direita)
			if (tecla=='P') {tecla='7';}                // Parado (seta para baixo)
			printf(" Enviado : %c\r",tecla);
			serial_envia(porta,para);
			serial_limpa(porta); 
			serial_envia(porta,tecla);
			serial_limpa(porta); 
			tecla="";
		}
	}
	fim:
	serial_fecha(porta);
	printf(" Programa encerrado!\n");
	printf(" Tecle algo para sair\n");
	getch();
	system("cls");
}

// ------------------------------------------------------------------------------
// A T E N   O  !
// ----------------
// NO ALTERE AS LINHAS ABAIXO 
// ------------------------------------------------------------------------------

/********************************************************************************
 Manipulador para inicializao da porta serial (abertura)
 ********************************************************************************/
HANDLE serial_abre (int porta, const long int BaudRate, const char parity, const char data) {
    BOOL bPortReady;
    DCB dcb;
	COMMTIMEOUTS CommTimeouts;
    char ComPortName[5];
	if (porta<10) {
		ComPortName[0]='C';ComPortName[1]='O';ComPortName[2]='M';
		ComPortName[3]='0'+ porta;
	} else {
    	if (porta<20) {
		   ComPortName[0]='C';ComPortName[1]='O';ComPortName[2]='M';ComPortName[3]='1';
		   ComPortName[4]='0'+ porta-10;
	    } else {
	        if (porta<30) {
		       ComPortName[0]='C';ComPortName[1]='O';ComPortName[2]='M';ComPortName[3]='2';
		       ComPortName[4]='0'+ porta-20;
	        }	
		}
	}
 
    printf("\n Abrindo porta %s\n\n",ComPortName);

    
    // Criao do manipulador da porta serial
    // **************************************
    hCom = CreateFile(ComPortName, GENERIC_READ | GENERIC_WRITE,
                                  0,            // acesso exclusivo
                                  NULL,         // sem segurana
                                  OPEN_EXISTING,
                                  0,            // sem sobreposio 
                                  NULL);        // sem template

    // Verificaes do manipulador
    // ***************************
    if ((int)hCom <= 0) {
        printf(" Falha na conexo (%s)- erro %d\n\r", ComPortName, GetLastError());
        return 0;
    }
    bPortReady = SetupComm(hCom, 128, 128); // set buffer sizes
    if (!bPortReady ) {    // verifica se houve erro na configurao da porta serial
        printf(" Falha de configurao (%s) - erro %d\n\r", ComPortName, GetLastError());
        return 0;
    }
    bPortReady = GetCommState(hCom, &dcb);
    if (!bPortReady ) {    // verifica se houve erro na leitura do estado da porta serial
        printf(" Falha de leitura (%s) - erro %d\n\r", ComPortName,  GetLastError());
        return 0;
    }
    
    // Configuraes gerais da porta serial
    // ************************************
    dcb.BaudRate = BaudRate;
    if( data == '7') dcb.ByteSize = 7;           // tamanho do byte = 7 bits
    else             dcb.ByteSize = 8;           // tamanho do byte = 8 bits
    if( parity == 'E') dcb.Parity = EVENPARITY;  // paridade par
    if( parity == 'O') dcb.Parity = ODDPARITY;   // paridade mpar
    else               dcb.Parity = NOPARITY;    // sem paridade
    dcb.StopBits = ONESTOPBIT;                   // bit de parada = 1
    dcb.fAbortOnError = TRUE;                    // habilita encerramento da comunicao mediante ocorrncia de erro
    dcb.fOutX = FALSE;                      // desabilita XON/XOFF (controle de fluxo por software) para transmisso
    dcb.fInX = FALSE;                       // desabilita XON/XOFF (controle de fluxo por software) para recepo
    dcb.fOutxCtsFlow = FALSE;               // desabilita o controle CTS (Clear to Send)
    dcb.fRtsControl = FALSE;                // desabilita o controle RTS (Request to Send)
    dcb.fOutxDsrFlow = FALSE;               // desabiliota o controle DSR (Data Set Ready)
    dcb.fDtrControl = DTR_CONTROL_DISABLE;  // desabilita o controle DTR (Data Terminal Ready)
    bPortReady = SetCommState(hCom, &dcb);  
    if (!bPortReady )  {                   // verifica se houve erro na configurao
        printf(" FALHA NA CONFIGURACAO - Porta Serial %s - erro %d\n\r", ComPortName,  GetLastError());
        return 0;
    }

    // Configurao dos temporizadores (timers)
    // ****************************************
    bPortReady = GetCommTimeouts (hCom, &CommTimeouts);
    CommTimeouts.ReadIntervalTimeout = 5 ;
    CommTimeouts.ReadTotalTimeoutConstant = 5 ;
    CommTimeouts.ReadTotalTimeoutMultiplier = 1 ;
    CommTimeouts.WriteTotalTimeoutConstant = 5 ;
    CommTimeouts.WriteTotalTimeoutMultiplier = 1 ;
    bPortReady = SetCommTimeouts (hCom, &CommTimeouts);
    if (!bPortReady ) {
        printf(" Falha de timeout (%s) - erro %d\n\r", ComPortName,  GetLastError());
        return 0;
    } else {
        printf(" COM%d conectada.", porta);
    }
    return (hCom); // Retorna o manipulador caso haja sucesso nas configuraes
}


/********************************************************************************
 Funo para fechamento da porta serial
********************************************************************************/
void serial_fecha(const int porta)
{
   CloseHandle(hCom);
}

/********************************************************************************
 Funo para leitura da porta serial (recebimento de 1 caractere)
********************************************************************************/
char serial_recebe(const int porta){
    char rxchar;
    BOOL bReadRC;
    static DWORD iBytesRead;
    bReadRC = ReadFile(hCom, &rxchar, 1, &iBytesRead, NULL);
    if (iBytesRead)    {
        return rxchar;    // retorna o caractere lido
    } else {
        return 0;         // retorna 0 se nenhum caractere for lido
    }
}

/********************************************************************************
 Funo para limpeza do buffer
********************************************************************************/
void serial_limpa(const int porta) {
    while(serial_recebe(porta)!=0)   ;
}

/********************************************************************************
 Funo para envio de caractere pela porta serial
********************************************************************************/
void serial_envia(const int porta, const int txchar) {
    BOOL bWriteRC;
    static DWORD iBytesWritten;
    bWriteRC = WriteFile(hCom, &txchar, 1, &iBytesWritten,NULL);
    return;
}

/********************************************************************************
 Funo de temporizao (timer)
********************************************************************************/
void delay_ms(clock_t millis) {
   clock_t endtime;
   endtime = millis + clock();
   while( endtime > clock() ) {;}
}


